"use strict";
var core_1 = require('@angular/core');
var parse_1 = require('../utils/parse');
var OrderByPipe = (function () {
    function OrderByPipe() {
        this.$parse = parse_1.Parse();
    }
    OrderByPipe.prototype.transform = function (array) {
        var _this = this;
        var attrs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            attrs[_i - 1] = arguments[_i];
        }
        // generate an array of predicate-objects contains
        // property getter, and descending indicator
        var predicates = attrs.map(function (pred) {
            var descending = pred.charAt(0) === '-' ? -1 : 1;
            pred = pred.replace(/^-/, '');
            return {
                getter: function (o) { return _this.$parse(pred)(o); },
                descend: descending
            };
        });
        // schwartzian transform idiom implementation. aka: "decorate-sort-undecorate"
        return array.map(function (item) {
            return {
                src: item,
                compareValues: predicates.map(function (predicate) { return predicate.getter(item); })
            };
        })
            .sort(function (o1, o2) {
            var i = -1, result = 0;
            while (++i < predicates.length) {
                if (o1.compareValues[i] < o2.compareValues[i])
                    result = -1;
                if (o1.compareValues[i] > o2.compareValues[i])
                    result = 1;
                if (result *= predicates[i].descend)
                    break;
            }
            return result;
        })
            .map(function (item) { return item.src; });
    };
    OrderByPipe.decorators = [
        { type: core_1.Pipe, args: [{
                    name: 'orderBy'
                },] },
    ];
    /** @nocollapse */
    OrderByPipe.ctorParameters = function () { return []; };
    return OrderByPipe;
}());
exports.OrderByPipe = OrderByPipe;
//# sourceMappingURL=order-by.pipe.js.map